@@ -92,12 +92,41 @@ func (*ErrWrapExpr) exprNode() {}
 
 // -----------------------------------------------------------------------------
 
-// ForPhrase represents `for k, v <- listOrMap`
+// LambdaExpr represents
+//    `(x, y, ...) => exprOrExprTuple`
+//    `x => exprOrExprTuple`
+//    `=> exprOrExprTuple`
+// here exprOrExprTuple represents
+//    `expr`
+//    `(expr1, expr2, ...)`
+type LambdaExpr struct {
+	First, Last token.Pos
+	Lhs         []*Ident
+	Rarrow      token.Pos
+	Rhs         []Expr
+	LhsHasParen bool
+	RhsHasParen bool
+}
+
+func (p *LambdaExpr) Pos() token.Pos {
+	return p.First
+}
+
+func (p *LambdaExpr) End() token.Pos {
+	return p.Last
+}
+
+func (*LambdaExpr) exprNode() {}
+
+// -----------------------------------------------------------------------------
+
+// ForPhrase represents `for k, v <- container, cond`
 type ForPhrase struct {
 	For        token.Pos // position of "for" keyword
 	Key, Value *Ident    // Key may be nil
 	TokPos     token.Pos // position of "<-" operator
-	X          Expr      // value to range over, must be list or map
+	X          Expr      // value to range over
+	Init       Stmt      // initialization statement; or nil
 	Cond       Expr      // value filter, can be nil
 }
 
@@ -110,10 +139,10 @@ func (p *ForPhrase) End() token.Pos { return p.X.End() }
 func (p *ForPhrase) exprNode() {}
 
 // ComprehensionExpr represents
-//    `[vexpr for k1, v1 <- listOrMap1, cond1 ...]` or
-//    `{vexpr for k1, v1 <- listOrMap1, cond1 ...}` or
-//    `{kexpr: vexpr for k1, v1 <- listOrMap1, cond1 ...}` or
-//    `{for k1, v1 <- listOrMap1, cond1 ...}` or
+//    `[vexpr for k1, v1 <- container1, cond1 ...]` or
+//    `{vexpr for k1, v1 <- container1, cond1 ...}` or
+//    `{kexpr: vexpr for k1, v1 <- container1, cond1 ...}` or
+//    `{for k1, v1 <- container1, cond1 ...}` or
 type ComprehensionExpr struct {
 	Lpos token.Pos   // position of "[" or "{"
 	Tok  token.Token // token.LBRACK '[' or token.LBRACE '{'
