@@ -127,6 +127,625 @@ func init() {
 `)
 }
 
+func TestTypeConvIssue804(t *testing.T) {
+	gopClTest(t, `
+c := make(chan int)
+d := (chan<- int)(c)
+e := (<-chan int)(c)
+f := (*int)(nil)
+a := c == d
+b := c == e
+`, `package main
+
+func main() {
+	c := make(chan int)
+	d := (chan<- int)(c)
+	e := (<-chan int)(c)
+	f := (*int)(nil)
+	a := c == d
+	b := c == e
+}
+`)
+}
+
+func TestUntypedFloatIssue798(t *testing.T) {
+	gopClTest(t, `
+func isPow10(x uint64) bool {
+	switch x {
+	case 1e0, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9,
+		1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19:
+		return true
+	}
+	return false
+}
+`, `package main
+
+func isPow10(x uint64) bool {
+	switch x {
+	case 1e0, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9, 1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19:
+		return true
+	}
+	return false
+}
+`)
+}
+
+func TestInterfaceIssue795(t *testing.T) {
+	gopClTest(t, `
+type I interface {
+	a(s string) I
+	b(s string) string
+}
+
+type T1 int
+
+func (t T1) a(s string) I {
+	return t
+}
+
+func (T1) b(s string) string {
+	return s
+}
+`, `package main
+
+type I interface {
+	a(s string) I
+	b(s string) string
+}
+type T1 int
+
+func (t T1) a(s string) I {
+	return t
+}
+func (T1) b(s string) string {
+	return s
+}
+`)
+}
+
+func TestChanRecvIssue789(t *testing.T) {
+	gopClTest(t, `
+func foo(ch chan int) (int, bool) {
+	x, ok := (<-ch)
+	return x, ok
+}
+`, `package main
+
+func foo(ch chan int) (int, bool) {
+	x, ok := <-ch
+	return x, ok
+}
+`)
+}
+
+func TestNamedChanCloseIssue790(t *testing.T) {
+	gopClTest(t, `
+type XChan chan int
+
+func foo(ch XChan) {
+	close(ch)
+}
+`, `package main
+
+type XChan chan int
+
+func foo(ch XChan) {
+	close(ch)
+}
+`)
+}
+
+func TestUntypedFloatIssue793(t *testing.T) {
+	gopClTest(t, `
+var a [1e1]int
+`, `package main
+
+var a [10]int
+`)
+}
+
+func TestUntypedFloatIssue788(t *testing.T) {
+	gopClTest(t, `
+func foo(v int) bool {
+    return v > 1.1e5
+}
+`, `package main
+
+func foo(v int) bool {
+	return v > 1.1e5
+}
+`)
+}
+
+func TestSwitchCompositeLitIssue801(t *testing.T) {
+	gopClTest(t, `
+type T struct {
+	X int
+}
+
+switch (T{}) {
+case T{1}:
+	panic("bad")
+}
+`, `package main
+
+type T struct {
+	X int
+}
+
+func main() {
+	switch (T{}) {
+	case T{1}:
+		panic("bad")
+	}
+}
+`)
+}
+
+func TestConstIssue800(t *testing.T) {
+	gopClTest(t, `
+const (
+	h0_0, h0_1 = 1.0 / (iota + 1), 1.0 / (iota + 2)
+	h1_0, h1_1
+)
+`, `package main
+
+const (
+	h0_0, h0_1 = 1.0 / (iota + 1), 1.0 / (iota + 2)
+	h1_0, h1_1
+)
+`)
+}
+
+func TestConstIssue805(t *testing.T) {
+	gopClTest(t, `
+const (
+	n1 = +5
+	d1 = +3
+
+	q1 = +1
+	r1 = +2
+)
+
+const (
+	ret1 = n1/d1 != q1
+	ret2 = n1%d1 != r1
+	ret3 = n1/d1 != q1 || n1%d1 != r1
+)
+`, `package main
+
+const (
+	n1 = +5
+	d1 = +3
+	q1 = +1
+	r1 = +2
+)
+const (
+	ret1 = n1/d1 != q1
+	ret2 = n1%d1 != r1
+	ret3 = false
+)
+`)
+}
+
+func TestUntypedNilIssue806(t *testing.T) {
+	gopClTest(t, `
+switch f := func() {}; f {
+case nil:
+}
+`, `package main
+
+func main() {
+	switch f := func() {
+	}; f {
+	case nil:
+	}
+}
+`)
+}
+
+func TestSwitchIssue807(t *testing.T) {
+	gopClTest(t, `
+switch {
+case interface{}(true):
+}
+`, `package main
+
+func main() {
+	switch {
+	case interface {
+	}(true):
+	}
+}
+`)
+}
+
+func TestUntypedComplexIssue799(t *testing.T) {
+	gopClTest(t, `
+const ulp1 = imag(1i + 2i / 3 - 5i / 3)
+const ulp2 = imag(1i + complex(0, 2) / 3 - 5i / 3)
+
+func main() {
+	const a = (ulp1 == ulp2)
+}
+`, `package main
+
+const ulp1 = imag(1i + 2i/3 - 5i/3)
+const ulp2 = imag(1i + complex(0, 2)/3 - 5i/3)
+
+func main() {
+	const a = ulp1 == ulp2
+}
+`)
+}
+
+func TestUnderscoreConstAndVar(t *testing.T) {
+	gopClTest(t, `
+const (
+	c0 = 1 << iota
+	_
+	_
+	_
+	c4
+)
+
+func i() int {
+	return 23
+}
+
+var (
+	_ = i()
+	_ = i()
+)
+`, `package main
+
+const (
+	c0 = 1 << iota
+	_
+	_
+	_
+	c4
+)
+
+func i() int {
+	return 23
+}
+
+var _ = i()
+var _ = i()
+`)
+}
+
+func TestUnderscoreFuncAndMethod(t *testing.T) {
+	gopClTest(t, `
+func _() {
+}
+
+type T struct {
+	_, _, _ int
+}
+
+func (T) _() {
+}
+
+func (T) _() {
+}
+`, `package main
+
+type T struct {
+	_ int
+	_ int
+	_ int
+}
+
+func (T) _() {
+}
+func (T) _() {
+}
+func _() {
+}
+`)
+}
+
+func TestErrWrapIssue772(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func t() (int,int,error){
+	return 0, 0, nil
+}
+
+func main() {
+	a, b := t()!
+	println(a, b)
+}`, `package main
+
+import fmt "fmt"
+
+func t() (int, int, error) {
+	return 0, 0, nil
+}
+func main() {
+	a, b := func() (_gop_ret int, _gop_ret2 int) {
+		var _gop_err error
+		_gop_ret, _gop_ret2, _gop_err = t()
+		if _gop_err != nil {
+			panic(_gop_err)
+		}
+		return
+	}()
+	fmt.Println(a, b)
+}
+`)
+}
+
+func TestErrWrapIssue778(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func t() error {
+	return nil
+}
+
+func main() {
+	t()!
+}`, `package main
+
+func t() error {
+	return nil
+}
+func main() {
+	func() {
+		var _gop_err error
+		_gop_err = t()
+		if _gop_err != nil {
+			panic(_gop_err)
+		}
+		return
+	}()
+}
+`)
+}
+
+func TestIssue774(t *testing.T) {
+	gopClTest(t, `
+package main
+
+import "fmt"
+
+func main() {
+	var a AA = &A{str: "hello"}
+	fmt.Println(a.(*A))
+}
+
+type AA interface {
+	String() string
+}
+
+type A struct {
+	str string
+}
+
+func (a *A) String() string {
+	return a.str
+}
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	var a AA = &A{str: "hello"}
+	fmt.Println(a.(*A))
+}
+
+type AA interface {
+	String() string
+}
+type A struct {
+	str string
+}
+
+func (a *A) String() string {
+	return a.str
+}
+`)
+	gopClTest(t, `
+package main
+
+import "fmt"
+
+func main() {
+	a := get()
+	fmt.Println(a.(*A))
+}
+
+func get()AA{
+	var a AA
+	return a
+}
+
+type AA interface {
+	String() string
+}
+
+type A struct {
+	str string
+}
+
+func (a *A) String() string {
+	return a.str
+}
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	a := get()
+	fmt.Println(a.(*A))
+}
+
+type AA interface {
+	String() string
+}
+
+func get() AA {
+	var a AA
+	return a
+}
+
+type A struct {
+	str string
+}
+
+func (a *A) String() string {
+	return a.str
+}
+`)
+}
+
+func TestBlockStmt(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func main() {
+	{
+		type T int
+		t := T(100)
+		println(t)
+	}
+	{
+		type T string
+		t := "hello"
+		println(t)
+	}
+}
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	{
+		type T int
+		t := T(100)
+		fmt.Println(t)
+	}
+	{
+		type T string
+		t := "hello"
+		fmt.Println(t)
+	}
+}
+`)
+}
+
+func TestConstTypeConvIssue792(t *testing.T) {
+	gopClTest(t, `
+const dots = ". . . " + ". . . . . "
+const n = uint(len(dots))
+`, `package main
+
+const dots = ". . . " + ". . . . . "
+const n = uint(len(dots))
+`)
+}
+
+func TestVarInitTwoValueIssue791(t *testing.T) {
+	gopClTest(t, `
+var (
+	m      = map[string]string{"a": "A"}
+	a, ok  = m["a"]
+)
+`, `package main
+
+var m = map[string]string{"a": "A"}
+var a, ok = m["a"]
+`)
+}
+
+func TestVarAfterMain(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func main() {
+	println(i)
+}
+
+var i int
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	fmt.Println(i)
+}
+
+var i int
+`)
+	gopClTest(t, `
+package main
+
+func f(v float64) float64 {
+	return v
+}
+func main() {
+	sink = f(100)
+}
+
+var sink float64
+`, `package main
+
+func f(v float64) float64 {
+	return v
+}
+func main() {
+	sink = f(100)
+}
+
+var sink float64
+`)
+}
+
+func TestVarAfterMain2(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func main() {
+	println(i)
+}
+
+var i = 100
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	fmt.Println(i)
+}
+
+var i = 100
+`)
+}
+
+func TestVarInMain(t *testing.T) {
+	gopClTest(t, `
+package main
+
+func main() {
+	v := []uint64{2, 3, 5}
+	var n = len(v)
+	println(n)
+}`, `package main
+
+import fmt "fmt"
+
+func main() {
+	v := []uint64{2, 3, 5}
+	var n = len(v)
+	fmt.Println(n)
+}
+`)
+}
+
 func TestSelect(t *testing.T) {
 	gopClTest(t, `
 
@@ -258,6 +877,26 @@ func foo(shape Shape) {
 `)
 }
 
+func TestInterfaceEmbedded(t *testing.T) {
+	gopClTest(t, `
+type Shape interface {
+	Area() float64
+}
+
+type Bar interface {
+	Shape
+}
+`, `package main
+
+type Shape interface {
+	Area() float64
+}
+type Bar interface {
+	Shape
+}
+`)
+}
+
 func TestInterfaceExample(t *testing.T) {
 	gopClTest(t, `
 type Shape interface {
@@ -1448,6 +2087,23 @@ func main() {
 `)
 }
 
+func TestIndexArrayPtrIssue784(t *testing.T) {
+	gopClTest(t, `
+type intArr [2]int
+
+func foo(a *intArr) {
+	a[1] = 10
+}
+`, `package main
+
+type intArr [2]int
+
+func foo(a *intArr) {
+	a[1] = 10
+}
+`)
+}
+
 func TestMemberVal(t *testing.T) {
 	gopClTest(t, `import "strings"
 
@@ -1467,6 +2123,32 @@ func main() {
 `)
 }
 
+func TestNamedPtrMemberIssue786(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+	req int
+}
+
+type pfoo *foo
+
+func bar(p pfoo) {
+	println(p.req)
+}
+`, `package main
+
+import fmt "fmt"
+
+type foo struct {
+	req int
+}
+type pfoo *foo
+
+func bar(p pfoo) {
+	fmt.Println(p.req)
+}
+`)
+}
+
 func TestMember(t *testing.T) {
 	gopClTest(t, `
 
@@ -1513,6 +2195,23 @@ func foo(a *int, b int) {
 `)
 }
 
+func TestNamedPtrIssue797(t *testing.T) {
+	gopClTest(t, `
+type Bar *int
+
+func foo(a Bar) {
+	var b int = *a
+}
+`, `package main
+
+type Bar *int
+
+func foo(a Bar) {
+	var b int = *a
+}
+`)
+}
+
 func TestMethod(t *testing.T) {
 	gopClTest(t, `
 type M int
@@ -1533,7 +2232,7 @@ type M int
 func (m M) Foo() {
 	fmt.Println("foo", m)
 }
-func ( M) Bar() {
+func (M) Bar() {
 	fmt.Println("bar")
 }
 `)
@@ -1564,7 +2263,7 @@ func ++(a foo) {
 
 var a, b foo
 var c = a - b
-var d = -a
+var d = -a       // TODO: -a have no return value!
 `, `package main
 
 import fmt "fmt"
@@ -1655,8 +2354,10 @@ const (
 var j int = i
 `, `package main
 
-const i = 1
-const x float64 = 1
+const (
+	i         = 1
+	x float64 = 1
+)
 
 var j int = i
 `)
@@ -1677,8 +2378,10 @@ func main() {
 import fmt "fmt"
 
 func main() {
-	const i = 1
-	const x float64 = 1
+	const (
+		i         = 1
+		x float64 = 1
+	)
 	var j int = i
 	fmt.Println("Hi")
 }
@@ -1689,8 +2392,10 @@ func TestIf(t *testing.T) {
 	gopClTest(t, `x := 0
 if t := false; t {
 	x = 3
-} else {
+} else if !t {
 	x = 5
+} else {
+	x = 7
 }
 println("x:", x)
 `, `package main
@@ -1701,8 +2406,10 @@ func main() {
 	x := 0
 	if t := false; t {
 		x = 3
-	} else {
+	} else if !t {
 		x = 5
+	} else {
+		x = 7
 	}
 	fmt.Println("x:", x)
 }
