@@ -49,7 +49,7 @@ func codeErrorTest(t *testing.T, msg, src string) {
 
 func TestErrErrWrap(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:2:2 undefined: a\nTODO: unbound type\n", `func main() {
+		"./bar.gop:2:2: undefined: a", `func main() {
 	a!
 }
 `)
@@ -57,38 +57,52 @@ func TestErrErrWrap(t *testing.T) {
 
 func TestErrVar(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:2:2 undefined: foo", `func main() {
+		"./bar.gop:6:5: assignment mismatch: 1 variables but fmt.Println returns 2 values", `import "fmt"
+
+func main() {
+}
+
+var a = fmt.Println(1)
+`)
+	codeErrorTest(t,
+		"./bar.gop:4:5: assignment mismatch: 1 variables but 2 values", `func main() {
+}
+
+var a = 1, 2
+`)
+	codeErrorTest(t,
+		"./bar.gop:2:2: undefined: foo", `func main() {
 	foo.x = 1
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:2 use of builtin len not in function call", `func main() {
+		"./bar.gop:2:2: use of builtin len not in function call", `func main() {
 	len.x = 1
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:10 undefined: foo", `func main() {
+		"./bar.gop:2:10: undefined: foo", `func main() {
 	println(foo.x)
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:9 use of builtin len not in function call", `func main() {
+		"./bar.gop:2:9: use of builtin len not in function call", `func main() {
 println(len.x)
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:10 undefined: foo", `func main() {
+		"./bar.gop:2:10: undefined: foo", `func main() {
 	println(foo)
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:3:20 use of builtin len not in function call", `package main
+		"./bar.gop:3:20: use of builtin len not in function call", `package main
 
 func foo(v map[int]len) {
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:5:20 bar is not a type", `package main
+		"./bar.gop:5:20: bar is not a type", `package main
 
 var bar = 1
 
@@ -96,48 +110,53 @@ func foo(v map[int]bar) {
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:6 use of builtin len not in function call", `func main() {
+		"./bar.gop:2:6: use of builtin len not in function call", `func main() {
 	new(len)
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:2 undefined: foo", `func main() {
+		"./bar.gop:2:2: undefined: foo", `func main() {
 	foo = 1
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:9 cannot use _ as value", `func main() {
+		"./bar.gop:2:9: cannot use _ as value", `func main() {
 	foo := _
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:2:9 use of builtin len not in function call", `func main() {
+		"./bar.gop:2:9: use of builtin len not in function call", `func main() {
 	foo := len
 }
+`)
+	codeErrorTest(t,
+		"./bar.gop:2:2: use of builtin println not in function call", `func main() {
+	println = "hello"
+}
 `)
 }
 
 func TestErrImport(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:5:2 cannot refer to unexported name os.undefined", `
+		"./bar.gop:5:2: cannot refer to unexported name os.undefined", `
 import "os"
 
 func foo() {
 	os.undefined
 }`)
 	codeErrorTest(t,
-		"./bar.gop:5:2 undefined: os.UndefinedObject", `
+		"./bar.gop:5:2: undefined: os.UndefinedObject", `
 import "os"
 
 func foo() {
 	os.UndefinedObject
 }`)
 	codeErrorTest(t,
-		"./bar.gop:2:13 undefined: testing", `
+		"./bar.gop:2:13: undefined: testing", `
 func foo(t *testing.T) {
 }`)
 	codeErrorTest(t,
-		"./bar.gop:4:12 testing.Verbose is not a type", `
+		"./bar.gop:4:12: testing.Verbose is not a type", `
 import "testing"
 
 func foo(t testing.Verbose) {
@@ -146,23 +165,30 @@ func foo(t testing.Verbose) {
 
 func TestErrConst(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:3:7 a redeclared in this block\n\tprevious declaration at ./bar.gop:2:5", `
+		"./bar.gop:3:7: a redeclared in this block\n\tprevious declaration at ./bar.gop:2:5", `
 var a int
 const a = 1
+`)
+	codeErrorTest(t,
+		"./bar.gop:4:2: missing value in const declaration", `
+const (
+	a = iota
+	b, c
+)
 `)
 }
 
 func TestErrNewVar(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:3:5 a redeclared in this block\n\tprevious declaration at ./bar.gop:2:5", `
+		"./bar.gop:3:5: a redeclared in this block\n\tprevious declaration at ./bar.gop:2:5", `
 var a int
 var a string
 `)
 }
 
 func TestErrDefineVar(t *testing.T) {
-	codeErrorTest(t, "./bar.gop:3:1 no new variables on left side of :=\n"+
-		"./bar.gop:3:6 cannot use \"Hi\" (type untyped string) as type int in assignment", `
+	codeErrorTest(t, "./bar.gop:3:1: no new variables on left side of :=\n"+
+		"./bar.gop:3:6: cannot use \"Hi\" (type untyped string) as type int in assignment", `
 a := 1
 a := "Hi"
 `)
@@ -170,7 +196,7 @@ a := "Hi"
 
 func TestErrAssign(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:8:1 assignment mismatch: 1 variables but bar returns 2 values`, `
+		`./bar.gop:8:1: assignment mismatch: 1 variables but bar returns 2 values`, `
 
 func bar() (n int, err error) {
 	return
@@ -180,7 +206,7 @@ x := 1
 x = bar()
 `)
 	codeErrorTest(t,
-		`./bar.gop:4:1 assignment mismatch: 1 variables but 2 values`, `
+		`./bar.gop:4:1: assignment mismatch: 1 variables but 2 values`, `
 
 x := 1
 x = 1, "Hi"
@@ -189,28 +215,28 @@ x = 1, "Hi"
 
 func TestErrReturn(t *testing.T) {
 	codeErrorTest(t,
-		"./bar.gop:4:2 too few arguments to return\n\thave (untyped int)\n\twant (int, error)", `
+		"./bar.gop:4:2: too few arguments to return\n\thave (untyped int)\n\twant (int, error)", `
 
 func foo() (int, error) {
 	return 1
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:4:2 too many arguments to return\n\thave (untyped int, untyped int, untyped string)\n\twant (int, error)", `
+		"./bar.gop:4:2: too many arguments to return\n\thave (untyped int, untyped int, untyped string)\n\twant (int, error)", `
 
 func foo() (int, error) {
 	return 1, 2, "Hi"
 }
 `)
 	codeErrorTest(t,
-		`./bar.gop:4:12 cannot use "Hi" (type untyped string) as type error in return argument`, `
+		`./bar.gop:4:12: cannot use "Hi" (type untyped string) as type error in return argument`, `
 
 func foo() (int, error) {
 	return 1, "Hi"
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:8:2 too few arguments to return\n\thave (byte)\n\twant (int, error)", `
+		"./bar.gop:8:2: too few arguments to return\n\thave (byte)\n\twant (int, error)", `
 
 func bar() (v byte) {
 	return
@@ -221,7 +247,7 @@ func foo() (int, error) {
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:8:2 too many arguments to return\n\thave (n int, err error)\n\twant (v byte)", `
+		"./bar.gop:8:2: too many arguments to return\n\thave (n int, err error)\n\twant (v byte)", `
 
 func bar() (n int, err error) {
 	return
@@ -232,7 +258,7 @@ func foo() (v byte) {
 }
 `)
 	codeErrorTest(t,
-		`./bar.gop:8:2 cannot use byte value as type error in return argument`, `
+		`./bar.gop:8:2: cannot use byte value as type error in return argument`, `
 
 func bar() (n int, v byte) {
 	return
@@ -243,7 +269,7 @@ func foo() (int, error) {
 }
 `)
 	codeErrorTest(t,
-		"./bar.gop:4:2 not enough arguments to return\n\thave ()\n\twant (byte)", `
+		"./bar.gop:4:2: not enough arguments to return\n\thave ()\n\twant (byte)", `
 
 func foo() byte {
 	return
@@ -253,7 +279,7 @@ func foo() byte {
 
 func TestErrForRange(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:4:8 cannot assign type string to a (type int) in range`, `
+		`./bar.gop:4:8: cannot assign type string to a (type int) in range`, `
 a := 1
 var b []string
 for _, a = range b {
@@ -263,7 +289,7 @@ for _, a = range b {
 
 func TestErrInitFunc(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:2:1 func init must have no arguments and no return values`, `
+		`./bar.gop:2:1: func init must have no arguments and no return values`, `
 func init(v byte) {
 }
 `)
@@ -271,7 +297,7 @@ func init(v byte) {
 
 func TestErrRecv(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:5:9 invalid receiver type a (a is a pointer type)`, `
+		`./bar.gop:5:9: invalid receiver type a (a is a pointer type)`, `
 
 type a *int
 
@@ -279,17 +305,17 @@ func (p a) foo() {
 }
 `)
 	codeErrorTest(t,
-		`./bar.gop:2:9 invalid receiver type error (error is an interface type)`, `
+		`./bar.gop:2:9: invalid receiver type error (error is an interface type)`, `
 func (p error) foo() {
 }
 `)
 	codeErrorTest(t,
-		`./bar.gop:2:9 invalid receiver type []byte ([]byte is not a defined type)`, `
+		`./bar.gop:2:9: invalid receiver type []byte ([]byte is not a defined type)`, `
 func (p []byte) foo() {
 }
 `)
 	codeErrorTest(t,
-		`./bar.gop:2:10 invalid receiver type []byte ([]byte is not a defined type)`, `
+		`./bar.gop:2:10: invalid receiver type []byte ([]byte is not a defined type)`, `
 func (p *[]byte) foo() {
 }
 `)
@@ -297,55 +323,63 @@ func (p *[]byte) foo() {
 
 func TestErrStructLit(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:39 too many values in struct{x int; y string}{...}`, `
+		`./bar.gop:3:39: too many values in struct{x int; y string}{...}`, `
 x := 1
 a := struct{x int; y string}{1, "Hi", 2}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:30 too few values in struct{x int; y string}{...}`, `
+		`./bar.gop:3:30: too few values in struct{x int; y string}{...}`, `
 x := 1
 a := struct{x int; y string}{1}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:33 cannot use x (type int) as type string in value of field y`, `
+		`./bar.gop:3:33: cannot use x (type int) as type string in value of field y`, `
 x := 1
 a := struct{x int; y string}{1, x}
 `)
 }
 
+func TestErrArray(t *testing.T) {
+	codeErrorTest(t,
+		`./bar.gop:3:8: non-constant array bound n`, `
+var n int
+var a [n]int
+`)
+}
+
 func TestErrArrayLit(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:14 cannot use a as index which must be non-negative integer constant`,
+		`./bar.gop:3:14: cannot use a as index which must be non-negative integer constant`,
 		`
 a := "Hi"
 b := [10]int{a: 1}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:20 array index 10 out of bounds [0:10]`,
+		`./bar.gop:3:20: array index 10 out of bounds [0:10]`,
 		`
 a := "Hi"
 b := [10]int{9: 1, 3}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:16 array index 1 out of bounds [0:1]`,
+		`./bar.gop:3:16: array index 1 out of bounds [0:1]`,
 		`
 a := "Hi"
 b := [1]int{1, 2}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:14 array index 12 (value 12) out of bounds [0:10]`,
+		`./bar.gop:3:14: array index 12 (value 12) out of bounds [0:10]`,
 		`
 a := "Hi"
 b := [10]int{12: 2}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:14 cannot use a+"!" (type string) as type int in array literal`,
+		`./bar.gop:3:14: cannot use a+"!" (type string) as type int in array literal`,
 		`
 a := "Hi"
 b := [10]int{a+"!"}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:17 cannot use a (type string) as type int in array literal`,
+		`./bar.gop:3:17: cannot use a (type string) as type int in array literal`,
 		`
 a := "Hi"
 b := [10]int{2: a}
@@ -354,19 +388,19 @@ b := [10]int{2: a}
 
 func TestErrSliceLit(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:12 cannot use a as index which must be non-negative integer constant`,
+		`./bar.gop:3:12: cannot use a as index which must be non-negative integer constant`,
 		`
 a := "Hi"
 b := []int{a: 1}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:12 cannot use a (type string) as type int in slice literal`,
+		`./bar.gop:3:12: cannot use a (type string) as type int in slice literal`,
 		`
 a := "Hi"
 b := []int{a}
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:15 cannot use a (type string) as type int in slice literal`,
+		`./bar.gop:3:15: cannot use a (type string) as type int in slice literal`,
 		`
 a := "Hi"
 b := []int{2: a}
@@ -375,8 +409,8 @@ b := []int{2: a}
 
 func TestErrMapLit(t *testing.T) {
 	codeErrorTest(t, // TODO: first column need correct
-		`./bar.gop:2:34 cannot use 1+2 (type untyped int) as type string in map key
-./bar.gop:3:27 cannot use "Go" + "+" (type untyped string) as type int in map value`,
+		`./bar.gop:2:34: cannot use 1+2 (type untyped int) as type string in map key
+./bar.gop:3:27: cannot use "Go" + "+" (type untyped string) as type int in map value`,
 		`
 a := map[string]int{1+2: 2}
 b := map[string]int{"Hi": "Go" + "+"}
@@ -385,20 +419,20 @@ b := map[string]int{"Hi": "Go" + "+"}
 
 func TestErrSlice(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:4:6 cannot slice a (type *byte)`,
+		`./bar.gop:4:6: cannot slice a (type *byte)`,
 		`
 var a *byte
 x := 1
 b := a[x:2]
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:6 cannot slice a (type bool)`,
+		`./bar.gop:3:6: cannot slice a (type bool)`,
 		`
 a := true
 b := a[1:2]
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:6 invalid operation a[1:2:5] (3-index slice of string)`,
+		`./bar.gop:3:6: invalid operation a[1:2:5] (3-index slice of string)`,
 		`
 a := "Hi"
 b := a[1:2:5]
@@ -407,13 +441,13 @@ b := a[1:2:5]
 
 func TestErrIndex(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:10 assignment mismatch: 2 variables but 1 values`,
+		`./bar.gop:3:10: assignment mismatch: 2 variables but 1 values`,
 		`
 a := "Hi"
 b, ok := a[1]
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:6 invalid operation: a[1] (type bool does not support indexing)`,
+		`./bar.gop:3:6: invalid operation: a[1] (type bool does not support indexing)`,
 		`
 a := true
 b := a[1]
@@ -422,7 +456,7 @@ b := a[1]
 
 func TestErrIndexRef(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:1 cannot assign to a[1] (strings are immutable)`,
+		`./bar.gop:3:1: cannot assign to a[1] (strings are immutable)`,
 		`
 a := "Hi"
 a[1] = 'e'
@@ -431,13 +465,13 @@ a[1] = 'e'
 
 func TestErrStar(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:2 invalid indirect of a (type string)`,
+		`./bar.gop:3:2: invalid indirect of a (type string)`,
 		`
 a := "Hi"
 *a = 'e'
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:7 invalid indirect of a (type string)`,
+		`./bar.gop:3:7: invalid indirect of a (type string)`,
 		`
 a := "Hi"
 b := *a
@@ -446,7 +480,7 @@ b := *a
 
 func TestErrMember(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:6 a.x undefined (type string has no field or method x)`,
+		`./bar.gop:3:6: a.x undefined (type string has no field or method x)`,
 		`
 a := "Hello"
 b := a.x
@@ -455,13 +489,13 @@ b := a.x
 
 func TestErrMemberRef(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:3:1 a.x undefined (type string has no field or method x)`,
+		`./bar.gop:3:1: a.x undefined (type string has no field or method x)`,
 		`
 a := "Hello"
 a.x = 1
 `)
 	codeErrorTest(t,
-		`./bar.gop:5:1 a.x undefined (type aaa has no field or method x)`,
+		`./bar.gop:5:1: a.x undefined (type aaa has no field or method x)`,
 		`
 type aaa byte
 
@@ -469,7 +503,7 @@ a := aaa(0)
 a.x = 1
 `)
 	codeErrorTest(t,
-		`./bar.gop:5:1 a.z undefined (type aaa has no field or method z)`,
+		`./bar.gop:5:1: a.z undefined (type aaa has no field or method z)`,
 		`
 type aaa struct {x int; y string}
 
@@ -477,7 +511,7 @@ a := aaa{}
 a.z = 1
 `)
 	codeErrorTest(t,
-		`./bar.gop:3:1 a.z undefined (type struct{x int; y string} has no field or method z)`,
+		`./bar.gop:3:1: a.z undefined (type struct{x int; y string} has no field or method z)`,
 		`
 a := struct{x int; y string}{}
 a.z = 1
@@ -486,8 +520,8 @@ a.z = 1
 
 func TestErrLabel(t *testing.T) {
 	codeErrorTest(t,
-		`./bar.gop:4:1 label foo already defined at ./bar.gop:2:1
-./bar.gop:2:1 label foo defined and not used`,
+		`./bar.gop:4:1: label foo already defined at ./bar.gop:2:1
+./bar.gop:2:1: label foo defined and not used`,
 		`x := 1
 foo:
 	i := 1
@@ -495,7 +529,7 @@ foo:
 	i++
 `)
 	codeErrorTest(t,
-		`./bar.gop:2:6 label foo is not defined`,
+		`./bar.gop:2:6: label foo is not defined`,
 		`x := 1
 goto foo`)
 }
