@@ -20,6 +20,7 @@ import (
 	"log"
 	"math/big"
 	"reflect"
+	"strconv"
 
 	goast "go/ast"
 	gotoken "go/token"
@@ -83,25 +84,25 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 	case *ast.StarExpr:
 		compileStarExpr(ctx, v)
 	case *ast.ArrayType:
-		ctx.cb.Typ(toArrayType(ctx, v))
+		ctx.cb.Typ(toArrayType(ctx, v), v)
 	case *ast.MapType:
-		ctx.cb.Typ(toMapType(ctx, v))
+		ctx.cb.Typ(toMapType(ctx, v), v)
 	case *ast.StructType:
-		ctx.cb.Typ(toStructType(ctx, v))
+		ctx.cb.Typ(toStructType(ctx, v), v)
 	case *ast.ChanType:
-		ctx.cb.Typ(toChanType(ctx, v))
+		ctx.cb.Typ(toChanType(ctx, v), v)
 	case *ast.InterfaceType:
-		ctx.cb.Typ(toInterfaceType(ctx, v))
+		ctx.cb.Typ(toInterfaceType(ctx, v), v)
 	case *ast.ComprehensionExpr:
 		compileComprehensionExpr(ctx, v, twoValue != nil && twoValue[0])
 	case *ast.TypeAssertExpr:
 		compileTypeAssertExpr(ctx, v, twoValue != nil && twoValue[0])
 	case *ast.ParenExpr:
-		compileExpr(ctx, v.X)
+		compileExpr(ctx, v.X, twoValue...)
 	case *ast.ErrWrapExpr:
 		compileErrWrapExpr(ctx, v)
 	case *ast.FuncType:
-		ctx.cb.Typ(toFuncType(ctx, v, nil))
+		ctx.cb.Typ(toFuncType(ctx, v, nil), v)
 	case *ast.Ellipsis:
 		panic("compileEllipsis: ast.Ellipsis unexpected")
 	case *ast.KeyValueExpr:
@@ -152,7 +153,7 @@ func compileTypeAssertExpr(ctx *blockCtx, v *ast.TypeAssertExpr, twoValue bool)
 		panic("TODO: x.(type) is only used in type switch")
 	}
 	typ := toType(ctx, v.Type)
-	ctx.cb.TypeAssert(typ, twoValue)
+	ctx.cb.TypeAssert(typ, twoValue, v)
 }
 
 func compileIndexExpr(ctx *blockCtx, v *ast.IndexExpr, twoValue bool) { // x[i]
@@ -174,7 +175,7 @@ func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
 func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		if compileClassMember(ctx, x, x.Name, false, true) {
+		if compileClassMember(ctx, x, false, true) {
 			break
 		}
 		o, at, builtin := lookupParent(ctx, x.Name)
@@ -205,12 +206,12 @@ func isBuiltin(o types.Object) bool {
 func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, autoCall bool) {
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		if compileClassMember(ctx, x, x.Name, autoCall, false) {
+		if compileClassMember(ctx, x, autoCall, false) {
 			break
 		}
 		o, at, builtin := lookupParent(ctx, x.Name)
 		if at != nil {
-			if compilePkgRef(ctx, at, v.Sel.Name, false) {
+			if compilePkgRef(ctx, at, v.Sel, false) {
 				return
 			}
 			if token.IsExported(v.Sel.Name) {
@@ -232,15 +233,15 @@ func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, autoCall bool) {
 	}
 }
 
-func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, name string, lhs bool) bool {
-	v := pkgRef(at, name)
+func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, name *ast.Ident, lhs bool) bool {
+	v := pkgRef(at, name.Name)
 	if v == nil {
 		return false
 	}
 	if lhs {
-		ctx.cb.VarRef(v)
+		ctx.cb.VarRef(v, name)
 	} else {
-		ctx.cb.Val(v)
+		ctx.cb.Val(v, name)
 	}
 	return true
 }
@@ -373,13 +374,16 @@ func compileIdentLHS(ctx *blockCtx, v *ast.Ident) {
 	if name == "_" {
 		ctx.cb.VarRef(nil)
 	} else {
-		if compileClassMember(ctx, v, name, false, true) {
+		if compileClassMember(ctx, v, false, true) {
 			return
 		}
-		_, o := ctx.cb.Scope().LookupParent(name, gotoken.NoPos)
+		o, _, builtin := lookupParent(ctx, name)
 		if o == nil {
 			panic(ctx.newCodeErrorf(v.Pos(), "undefined: %s", name))
 		}
+		if isBuiltin(builtin) {
+			panic(ctx.newCodeErrorf(v.Pos(), "use of builtin %s not in function call", name))
+		}
 		ctx.cb.VarRef(o, v)
 	}
 }
@@ -389,7 +393,7 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, allowBuiltin, autoCall bool)
 	if name == "_" {
 		panic(ctx.newCodeError(ident.Pos(), "cannot use _ as value"))
 	}
-	if compileClassMember(ctx, ident, name, autoCall, false) {
+	if compileClassMember(ctx, ident, autoCall, false) {
 		return
 	}
 	if o, _, builtin := lookupParent(ctx, name); o != nil {
@@ -402,17 +406,17 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, allowBuiltin, autoCall bool)
 	}
 }
 
-func compileClassMember(ctx *blockCtx, v ast.Node, name string, autoCall, lhs bool) bool {
+func compileClassMember(ctx *blockCtx, v *ast.Ident, autoCall, lhs bool) bool {
 	if ctx.fileType > 0 {
 		if fn := ctx.cb.Func(); fn != nil {
 			sig := fn.Type().(*types.Signature)
 			if recv := sig.Recv(); recv != nil {
 				ctx.cb.Val(recv)
-				if compileMember(ctx, v, name, autoCall, lhs) == nil {
+				if compileMember(ctx, v, v.Name, autoCall, lhs) == nil {
 					return true
 				}
 				ctx.cb.InternalStack().PopN(1)
-				if compilePkgRef(ctx, ctx.spx, name, lhs) {
+				if compilePkgRef(ctx, ctx.spx, v, lhs) {
 					return true
 				}
 			}
@@ -450,9 +454,6 @@ func lookupParent(ctx *blockCtx, name string) (types.Object, *gox.PkgRef, types.
 		}
 		return obj, nil, o
 	}
-	if debugLookup && o != nil {
-		log.Println("==> Lookup (Universe)", name)
-	}
 	return o, nil, o
 }
 
@@ -575,7 +576,7 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type
 	var kind = checkCompositeLitElts(ctx, v.Elts)
 	if v.Type != nil {
 		typ = toType(ctx, v.Type)
-		underlying = typ.Underlying()
+		underlying = getUnderlying(ctx, typ)
 	} else if expected != nil {
 		if t, ok := expected.(*types.Pointer); ok {
 			expected, hasPtr = t.Elem(), true
@@ -749,18 +750,45 @@ func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr) {
 	if !useClosure && (cb.Scope().Parent() == types.Universe) {
 		panic("TODO: can't use expr? in global")
 	}
-	ret := pkg.NewAutoParam("_gop_ret")
-	sig := types.NewSignature(nil, nil, types.NewTuple(ret), false)
+
+	compileExpr(ctx, v.X)
+	x := cb.InternalStack().Pop()
+	n := 0
+	results, ok := x.Type.(*types.Tuple)
+	if ok {
+		n = results.Len() - 1
+	}
+
+	var ret []*types.Var
+	if n > 0 {
+		i, retName := 0, "_gop_ret"
+		ret = make([]*gox.Param, n)
+		for {
+			ret[i] = pkg.NewAutoParam(retName)
+			i++
+			if i >= n {
+				break
+			}
+			retName = "_gop_ret" + strconv.Itoa(i+1)
+		}
+	}
+	sig := types.NewSignature(nil, nil, types.NewTuple(ret...), false)
 	if useClosure {
 		cb.NewClosureWith(sig).BodyStart(pkg)
 	} else {
 		cb.CallInlineClosureStart(sig, 0, false)
 	}
+
 	cb.NewVar(tyError, "_gop_err")
 	err := cb.Scope().Lookup("_gop_err")
-	cb.VarRef(ret).VarRef(err)
-	compileExpr(ctx, v.X)
-	cb.Assign(2, 1)
+
+	for _, retVar := range ret {
+		cb.VarRef(retVar)
+	}
+	cb.VarRef(err)
+	cb.InternalStack().Push(x)
+	cb.Assign(n+1, 1)
+
 	cb.If().Val(err).CompareNil(gotoken.NEQ).Then()
 	if v.Tok == token.NOT { // expr!
 		cb.Val(pkg.Builtin().Ref("panic")).Val(err).Call(1).EndStmt() // TODO: wrap err
