@@ -105,7 +105,10 @@ func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
 	case *ast.LabeledStmt:
 		compileLabeledStmt(ctx, v)
 	case *ast.BlockStmt:
+		ctx.cb.Block()
 		compileStmts(ctx, v.List)
+		ctx.cb.End()
+		return
 	case *ast.SelectStmt:
 		compileSelectStmt(ctx, v)
 	case *ast.EmptyStmt:
@@ -315,9 +318,13 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 	compileExpr(ctx, v.Cond)
 	cb.Then()
 	compileStmts(ctx, v.Body.List)
-	if v.Else != nil {
+	if e := v.Else; e != nil {
 		cb.Else()
-		compileStmt(ctx, v.Else)
+		if stmts, ok := e.(*ast.BlockStmt); ok {
+			compileStmts(ctx, stmts.List)
+		} else {
+			compileStmt(ctx, e)
+		}
 	}
 	cb.SetComments(comments, true)
 	cb.End()
@@ -509,16 +516,12 @@ func compileDeclStmt(ctx *blockCtx, expr *ast.DeclStmt) {
 				compileType(ctx, spec.(*ast.TypeSpec))
 			}
 		case token.CONST:
-			for _, spec := range d.Specs {
-				v := spec.(*ast.ValueSpec)
-				names := makeNames(v.Names)
-				loadConsts(ctx, names, v) // TODO: check bug
-			}
+			cdecl := ctx.pkg.NewConstDecl(ctx.cb.Scope())
+			loadConstSpecs(ctx, cdecl, d.Specs)
 		case token.VAR:
 			for _, spec := range d.Specs {
 				v := spec.(*ast.ValueSpec)
-				names := makeNames(v.Names)
-				loadVars(ctx, names, v) // TODO: check bug
+				loadVars(ctx, v, false)
 			}
 		default:
 			log.Panicln("TODO: compileDeclStmt - unknown")
