@@ -20,23 +20,23 @@ package install
 import (
 	"fmt"
 	"os"
-	"os/exec"
 	"strings"
 
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/cmd/gengo"
 	"github.com/goplus/gop/cmd/internal/base"
+	"github.com/goplus/gox"
 )
 
 // Cmd - gop install
 var Cmd = &base.Command{
-	UsageLine: "gop install [-v] <gopSrcDir|gopSrcFile>",
+	UsageLine: "gop install [-v] <GopPackages>",
 	Short:     "Build Go+ files and install target to GOBIN",
 }
 
 var (
 	flag        = &Cmd.Flag
-	flagVerbose = flag.Bool("v", false, "print the names of packages as they are compiled.")
+	flagVerbose = flag.Bool("v", false, "print verbose information")
 )
 
 func init() {
@@ -44,23 +44,29 @@ func init() {
 }
 
 func runCmd(cmd *base.Command, args []string) {
-	flag.Parse(args)
-	if flag.NArg() < 1 {
-		cmd.Usage(os.Stderr)
-		return
+	flag.Parse(base.SkipSwitches(args, flag))
+	ssargs := flag.Args()
+	if len(ssargs) == 0 {
+		ssargs = []string{"."}
 	}
-	var exitCode int
 	var recursive bool
-	var dir = flag.Arg(0)
+	var dir = ssargs[0]
 	if strings.HasSuffix(dir, "/...") {
 		dir = dir[:len(dir)-4]
 		recursive = true
 	}
 
+	if *flagVerbose {
+		gox.SetDebug(gox.DbgFlagAll &^ gox.DbgFlagComments)
+		cl.SetDebug(cl.DbgFlagAll)
+		cl.SetDisableRecover(true)
+	}
+	hasError := false
 	runner := new(gengo.Runner)
 	runner.SetAfter(func(p *gengo.Runner, dir string, flags int) error {
 		errs := p.ResetErrors()
 		if errs != nil {
+			hasError = true
 			for _, err := range errs {
 				fmt.Fprintln(os.Stderr, err)
 			}
@@ -68,21 +74,13 @@ func runCmd(cmd *base.Command, args []string) {
 		}
 		return nil
 	})
-	runner.GenGo(dir, recursive, &cl.Config{CacheLoadPkgs: *flagVerbose})
-	goCmd(dir, "install", args...)
-	os.Exit(exitCode)
-}
-
-func goCmd(dir string, op string, args ...string) error {
-	opwargs := make([]string, len(args)+1)
-	opwargs[0] = op
-	copy(opwargs[1:], args)
-	cmd := exec.Command("go", opwargs...)
-	cmd.Dir = dir
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
-	cmd.Env = os.Environ()
-	return cmd.Run()
+	baseConf := &cl.Config{PersistLoadPkgs: true}
+	runner.GenGo(dir, recursive, baseConf.Ensure())
+	if hasError {
+		os.Exit(1)
+	}
+	baseConf.PkgsLoader.Save()
+	base.RunGoCmd(dir, "install", args...)
 }
 
 // -----------------------------------------------------------------------------
